@@ -5,9 +5,14 @@ import sys
 from json import JSONDecodeError
 from typing import Dict
 
-from . import Command, BaseHandler
-from .commands import RunHttpFileHandler, FormatHttpFileHandler, GetNameReferencesHandler, ImportPostmanCollection, \
-    ContentExecuteHandler, ParseHttpData, ContentNameReferencesHandler
+from .handlers.basic_handlers import RunHttpFileHandler, ContentExecuteHandler, FormatHttpFileHandler, \
+    GetNameReferencesHandler, ContentNameReferencesHandler
+from .handlers.gohandler import TypeFromPos
+from .handlers.har2httphandler import Har2HttpHandler
+from .handlers.http2har import Http2Har
+from .handlers.http2postman import Http2Postman
+from .handlers.postman2http import ImportPostmanCollection
+from .models import Command, BaseHandler
 
 logger = logging.getLogger('handler')
 
@@ -15,8 +20,11 @@ handlers: Dict[str, BaseHandler] = {handler.get_method(): handler for handler in
                                     (FormatHttpFileHandler(), RunHttpFileHandler(), GetNameReferencesHandler(),
                                      ImportPostmanCollection(),
                                      ContentExecuteHandler(),
-                                     ParseHttpData(),
-                                     ContentNameReferencesHandler()
+                                     Http2Har(),
+                                     ContentNameReferencesHandler(),
+                                     TypeFromPos(),
+                                     Har2HttpHandler(),
+                                     Http2Postman(),
                                      )}
 
 
