@@ -3,21 +3,23 @@ import logging
 import os
 from http.cookiejar import LWPCookieJar
 from typing import Union
+from urllib.parse import urlparse, unquote, urlunparse
 
 import jstyleson as json
 from requests import PreparedRequest, Session, Response
 # this is bad, loading private stuff. find a better way
-from requests.auth import HTTPBasicAuth, HTTPDigestAuth
+from requests.auth import HTTPBasicAuth, HTTPDigestAuth, CONTENT_TYPE_FORM_URLENCODED
 from requests.status_codes import _codes as status_code
 from requests_pkcs12 import Pkcs12Adapter
 from textx import metamodel_from_file
 
-from . import eprint, Config, HttpDefBase, js3py
+from dothttp import APPLICATION_JSON, MIME_TYPE_JSON, UNIX_SOCKET_SCHEME, TEXT_PLAIN, CONTENT_TYPE
+from . import eprint, Config, HttpDefBase, js3py, AWS4Auth
 from .curl_utils import to_curl
 from .dsl_jsonparser import json_or_array_to_json
 from .json_utils import JSONEncoder
 from .parse_models import Allhttp
-from .utils import quote_or_unquote
+from .utils import quote_or_unquote, apply_quote_or_unquote
 
 JSON_ENCODER = JSONEncoder(indent=4)
 
@@ -25,20 +27,16 @@ try:
     import magic
 except ImportError:
     magic = None
-
 try:
-    import magic
-except ImportError:
-    magic = None
+    import requests_unixsocket
 
-FORM_URLENCODED = "application/x-www-form-urlencoded"
+    # will be able to make unix socket apis
+    requests_unixsocket.monkeypatch()
+except ImportError:
+    # in wasm phase, it will not be available, and can be ignored
+    pass
 
 DOTHTTP_COOKIEJAR = os.path.expanduser('~/.dothttp.cookiejar')
-
-MIME_TYPE_JSON = "application/json"
-
-CONTENT_TYPE = 'content-type'
-
 base_logger = logging.getLogger("dothttp")
 request_logger = logging.getLogger("request")
 curl_logger = logging.getLogger("curl")
@@ -53,8 +51,16 @@ dothttp_model = metamodel_from_file(dir_path)
 # noinspection PyPackageRequirements
 
 
+def get_new_session():
+    session = Session()
+    if requests_unixsocket:
+        from requests_unixsocket.adapters import UnixAdapter
+        session.mount('http+unix://', UnixAdapter())
+    return session
+
+
 class RequestBase(HttpDefBase):
-    global_session = Session()
+    global_session = get_new_session()
 
     def __init__(self, args: Config):
         super().__init__(args)
@@ -93,7 +99,7 @@ class RequestBase(HttpDefBase):
         if self.httpdef.session_clear:
             # calle should close session
             # TODO
-            return Session()
+            return get_new_session()
         session = self.global_session
         if not self.args.no_cookie:
             if cookie := self.get_cookie():
@@ -103,25 +109,12 @@ class RequestBase(HttpDefBase):
 
     @functools.lru_cache
     def get_request(self):
-        prep = self.get_request_notbody()
-        payload = self.httpdef.payload
-        prep.prepare_body(data=payload.data, json=payload.json, files=payload.files)
-        # prep.prepare_hooks({"response": self.save_cookie_call_back})
-        if payload.header and CONTENT_TYPE not in prep.headers:
-            # if content-type is provided by header
-            # we will not wish to update it
-            prep.headers[CONTENT_TYPE] = payload.header
-        request_logger.debug(f'request prepared completely {prep}')
-        return prep
-
-    def get_request_notbody(self):
+        # httpdef has to be loaded
+        # according to httpdef, prepared_request is built
         self.load_def()
-        prep = PreparedRequest()
-        prep.prepare_url(self.httpdef.url, self.httpdef.query)
-        prep.prepare_method(self.httpdef.method)
-        prep.prepare_headers(self.httpdef.headers)
+        prep = self.httpdef.get_prepared_request()
+        # cookie is separately prepared
         prep.prepare_cookies(self.get_cookie())
-        prep.prepare_auth(self.httpdef.auth, prep.url)
         return prep
 
     def run(self):
@@ -141,7 +134,7 @@ class CurlCompiler(RequestBase):
         curl_logger.debug(f'curl request generation completed successfully')
 
     def get_curl_output(self):
-        prep = self.get_request_notbody()
+        prep = self.get_request()
         parts = []
         payload = self.httpdef.payload
         if auth := self.httpdef.auth:
@@ -150,6 +143,12 @@ class CurlCompiler(RequestBase):
                 parts.append(("--user", f"{auth.username}:{auth.password}"))
             elif isinstance(auth, HTTPBasicAuth):
                 parts.append(("--user", f"{auth.username}:{auth.password}"))
+            elif isinstance(auth, AWS4Auth):
+                for header_key, header_value in prep.headers.items():
+                    # include only aws headers
+                    if header_key.startswith("x-amz") or header_key.lower().startswith("authorization"):
+                        parts += [('-H', '{0}: {1}'.format(header_key, header_value))]
+
         if certificate := self.httpdef.certificate:
             parts.append(("--cert", f"{certificate[0]}"))
             if self.httpdef.certificate[1]:
@@ -161,25 +160,52 @@ class CurlCompiler(RequestBase):
             parts.append(("--cert-type", f"P12"))
         if self.httpdef.allow_insecure:
             parts.append(("-k", None))
+        payload_parts = []
         if self.http.payload:
+            contenttype = None
             if self.http.payload.file:
-                parts.append(('--data', "@" + self.get_updated_content(self.http.payload.file)))
+                payload_parts += [('--data', "@" + self.get_updated_content(self.http.payload.file))]
             elif self.http.payload.fileswrap:
                 if payload.files:
                     for file in payload.files:
                         if isinstance(file[1][1], str):
-                            parts.append(('--form', file[0] + "=" + file[1][1]))
+                            payload_parts.append(('--form', file[0] + "=" + file[1][1]))
                         else:
-                            parts.append(('--form', file[0] + "=@" + file[1][1].name))
+                            payload_parts.append(('--form', file[0] + "=@" + file[1][1].name))
             elif self.http.payload.json:
                 dumps = json.dumps(payload.json, indent=4)
-                prep.headers["content-type"] = "application/json"
-                prep.prepare_body(data=dumps, files=None)
+                contenttype = APPLICATION_JSON
+                payload_parts += [('-d', dumps)]
+            elif self.http.payload.datajson:
+                payload_parts += [('-d', prep.body)]
+                contenttype = CONTENT_TYPE_FORM_URLENCODED
             else:
-                if payload.header:
-                    prep.headers['content-type'] = payload.header
-                prep.prepare_body(data=payload.data, json=payload.json, files=payload.files)
-        curl_req = to_curl(prep, parts)
+                payload_parts += [('-d', payload.data)]
+                contenttype = TEXT_PLAIN
+            if contenttype and CONTENT_TYPE not in self.httpdef.headers:
+                self.httpdef.headers['content-type'] = payload.header
+        # there few headers which set dynamically (basically auth)
+        # so set headers in the end
+        # if isinstance(self.httpdef.headers, AWS4Auth):
+        for k, v in sorted(self.httpdef.headers.items()):
+            parts += [('-H', '{0}: {1}'.format(k, v))]
+        url = prep.url
+
+        if url.startswith(UNIX_SOCKET_SCHEME):
+            scheme, netloc, path, params, query, fragment = urlparse(url)
+            unix_domain_socket_path = unquote(netloc)
+            parts.append(("--unix-socket", unix_domain_socket_path))
+            """
+                >>urlparse.urlparse("http://some.page.pl/nothing.py;someparam=some;otherparam=other?query1=val1&query2=val2#frag")
+                ParseResult(scheme='http', netloc='some.page.pl', path='/nothing.py', params='someparam=some;otherparam=other', query='query1=val1&query2=val2', fragment='frag')
+            """
+            # scheme will be changed to 'http:/'
+            # netlock will be added by '--unix-socket /var/run/docker.sock'
+            #
+            url = urlunparse(["http", "localhost", path, params, query, fragment])
+
+        parts += payload_parts
+        curl_req = to_curl(url=url, method=prep.method, bodydata=parts)
         return curl_req
 
 
@@ -204,26 +230,55 @@ class HttpFileFormatter(RequestBase):
                 output_str += new_line
             if http.namewrap and http.namewrap.name:
                 quote_type, name = quote_or_unquote(http.namewrap.name)
-                output_str += f"@name({quote_type}{name}{quote_type}){new_line}"
+                output_str += f"@name({quote_type}{name}{quote_type})"
+                output_str += f" : {apply_quote_or_unquote(http.namewrap.base)}{new_line}" if http.namewrap.base else new_line
+            if http.extra_args:
+                for extra_arg in http.extra_args:
+                    if extra_arg.clear:
+                        output_str += f"@clear{new_line}"
+                    if extra_arg.insecure:
+                        output_str += f"@insecure{new_line}"
             method = http.urlwrap.method if http.urlwrap.method else "GET"
             output_str += f'{method} "{http.urlwrap.url}"'
+            if certificate := http.certificate:
+                if hasattr(certificate, "cert") and certificate.cert:
+                    if certificate.cert and certificate.key:
+                        output_str += f'{new_line}certificate(cert={apply_quote_or_unquote(certificate.cert)}, key={apply_quote_or_unquote(certificate.key)})'
+                    elif certificate.cert:
+                        output_str += f'{new_line}certificate(cert={apply_quote_or_unquote(certificate.cert)})'
+                if hasattr(certificate, "p12_file") and certificate.p12_file:
+                    if certificate.p12_file and certificate.password:
+                        output_str += f'{new_line}p12(file={apply_quote_or_unquote(certificate.p12_file)}, password={apply_quote_or_unquote(certificate.password)})'
+                    else:
+                        output_str += f'{new_line}p12(file={apply_quote_or_unquote(certificate.p12_file)})'
             if auth_wrap := http.authwrap:
                 if basic_auth := auth_wrap.basic_auth:
                     output_str += f'{new_line}basicauth("{basic_auth.username}", "{basic_auth.password}")'
                 elif digest_auth := auth_wrap.digest_auth:
                     output_str += f'{new_line}digestauth("{digest_auth.username}", "{digest_auth.password}")'
+                elif aws_auth := auth_wrap.aws_auth:
+                    if aws_auth.service and aws_auth.region:
+                        output_str += f'{new_line}awsauth(access_id="{aws_auth.access_id}", secret_key="{aws_auth.secret_token}", service="{aws_auth.service}", region="{aws_auth.region}")'
+                    elif aws_auth.service:
+                        output_str += f'{new_line}awsauth(access_id="{aws_auth.access_id}", secret_key="{aws_auth.secret_token}", service="{aws_auth.service}")'
+                    else:
+                        output_str += f'{new_line}awsauth(access_id="{aws_auth.access_id}", secret_key="{aws_auth.secret_token}" )'
             if lines := http.lines:
                 def check_for_quotes(line):
                     quote_type, value = quote_or_unquote(line.header.value)
                     return f'"{line.header.key}": {quote_type}{value}{quote_type}'
 
-                headers = new_line.join(map(check_for_quotes, filter(lambda line: line.header, lines)))
+                headers = new_line.join(map(check_for_quotes,
+                                            filter(lambda
+                                                       line: line.header and line.header.value and line.header.key and type(
+                                                line.header.value) == str,
+                                                   lines)))
                 if headers:
                     output_str += f"\n{headers}"
 
                 query = new_line.join(map(query_to_http,
                                           filter(lambda line:
-                                                 line.query, lines)))
+                                                 line.query and type(line.query.value) == str, lines)))
                 if query:
                     output_str += f'\n{query}'
             if payload := http.payload:
@@ -308,7 +363,6 @@ class RequestCompiler(RequestBase):
     def get_response(self):
         session = self.get_session()
         request = self.get_request()
-        adaptor = None
         if self.httpdef.p12:
             session.mount(request.url,
                           Pkcs12Adapter(pkcs12_filename=self.httpdef.p12[0],
